<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0"
 				xmlns:wse="http://www.ibm.com/xmlns/prod/websphere/sensorevents/wse70"
				xmlns="http://wbe.ibm.com/6.2/Event/WSE_7_0">
							
				
	<xsl:param name="system">WebSphere Sensor Events 7.0</xsl:param>
	<xsl:param name="timestamp">2010-03-26T16:20:59Z</xsl:param>
	<xsl:param name="loginfo">This event generated by WSE 7.0</xsl:param>
	<xsl:param name="wbeevent">Asset_Observation</xsl:param>
	<xsl:param name="wbeobjectprefix">Asset_Observation</xsl:param>
	<xsl:param name="wbecustomnamespaces">false</xsl:param>
	<xsl:param name="transform_for_rules">false</xsl:param>	
	<xsl:param name="trace">false</xsl:param>
		      
	<!-- root template  -->
	<xsl:template match="/">
	
	    <xsl:if test="$trace='true'">
       		<xsl:message >Starting to process XML</xsl:message>
        </xsl:if>
        
        <!-- Check to see if the user specified a custom name space. if not, we can generate the default connector element -->
        <xsl:choose>
	      <xsl:when test="$wbecustomnamespaces='false'">
	      
		    <connector xmlns="http://wbe.ibm.com/6.2/Event/WSE_7_0" 
    		   xmlns:wse="http://www.ibm.com/xmlns/prod/websphere/sensorevents/wse70" 
    		   xmlns:wse_ext="http://www.ibm.com/xmlns/prod/websphere/sensorevents/wse70/wse_ext"
    		   xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
      		   name="WebSphere Sensor Events 7.0" version="6.2">
      		    
      		    
      		     <!-- Now generate the body of the connector element   -->
	             <xsl:call-template name="connector_body"/>    
	                
			</connector>
		  </xsl:when>
		  
	      <xsl:otherwise>
	      
	        <!-- If the user did specify a custom name space, the connector element is generated in the WBE Agent 
	             so we only need to generate the body -->
	        <xsl:call-template name="connector_body"/>
	        
	      </xsl:otherwise>
	      
	    </xsl:choose>
	   
	    
	</xsl:template>
	
	<xsl:template name="connector_body">
	
	   <xsl:if test="$trace='true'">
	   		<xsl:message >Building Connector Body</xsl:message>
	    </xsl:if>
	       	   
	       	  <!-- Now build the sensor event -->      	      
		      <xsl:for-each select="//wse:sensor_event">
		         <xsl:call-template name="sensor_event"/>
		      </xsl:for-each>
		     
		       <!-- Now build the sensor command -->     
		      <xsl:for-each select="//wse:sensor_command">
		     	<xsl:call-template name="sensor_command"/>
		      </xsl:for-each>
		      
		       <!-- Add the extra elements that WBE requires -->     
		      <xsl:element name="system">	      
		      	<xsl:value-of select="$system"/>
		      </xsl:element>
		      <xsl:element name="timestamp">
		  	  	<xsl:value-of select="$timestamp"/>
		  	  </xsl:element>
		  	  <xsl:element name="loginfo">
		  	    <xsl:value-of select="$loginfo"/>
		  	  </xsl:element>
		  	  
		  	  <xsl:if test="$trace='true'">
	       			<xsl:message >Connector Body Built</xsl:message>
	          </xsl:if>
	
	</xsl:template>
  	 
  	 <!-- Sensor Event Node -->         
	<xsl:template name="sensor_event">
	    
	    <xsl:if test="$trace='true'">
       		<xsl:message >Start building Sensor Event Node</xsl:message>
        </xsl:if>
        
        <xsl:element name="connector-bundle" >
	          <xsl:attribute name="name"><xsl:value-of select="$wbeevent"/></xsl:attribute>
	          <xsl:attribute name="type">Event</xsl:attribute>    
	  		  	
	  		 <!-- Add the extra elements that WBE requires -->  	 
	  		<xsl:apply-templates select="parent::node()" />
	  		
	  		<xsl:if test="$trace='true'">
       			<xsl:message >About to build Header</xsl:message>
        	</xsl:if>
        	 <!-- Generate the header for the sensor event -->  
    		<xsl:call-template name="create_header">
    			<xsl:with-param name="header"><xsl:value-of select="concat($wbeobjectprefix,'_')"/></xsl:with-param>
    		</xsl:call-template>
    		
    		<xsl:if test="$trace='true'">
       			<xsl:message >About to build Subject</xsl:message>
        	</xsl:if>
        	 <!-- Generating the subject for the sensor event -->  
    		<xsl:variable name = "subject" select="concat($wbeobjectprefix,'_Subject')" />
    		<xsl:for-each select="wse:subject/.">
    		  <xsl:element name = "{$subject}" > 
    			<xsl:copy-of select="*" />
    			<xsl:for-each select="wse:field">
	          	 	<xsl:call-template name="insert_field"/>  
	       		</xsl:for-each>
    		  </xsl:element>
    		</xsl:for-each>
    		
    		<xsl:if test="$trace='true'">
       			<xsl:message >About to build Location</xsl:message>
        	</xsl:if>
        	 <!-- Generating the location for the sensor event -->  
    		<xsl:variable name = "location" select="concat($wbeobjectprefix,'_Location')" />
    		<xsl:for-each select="wse:location/.">
    		  <xsl:element name = "{$location}" >  
    			<xsl:copy-of select="*" />
    			<xsl:for-each select="wse:field">
	          	 	<xsl:call-template name="insert_field"/>  
	        	</xsl:for-each>
    		  </xsl:element>
    		</xsl:for-each>
    		
    		<xsl:if test="$trace='true'">
       			<xsl:message >About to build Condition</xsl:message>
        	</xsl:if>
        	 <!-- Generating the condition for the sensor event -->  		
    		<xsl:variable name = "condition" select="concat($wbeobjectprefix,'_Condition')" />
    		<xsl:for-each select="wse:condition/.">  
    		  <xsl:element name = "{$condition}" >  
    			<xsl:copy-of select="*" />
    			<xsl:for-each select="wse:field">
	          	 	<xsl:call-template name="insert_field"/>  
	        	</xsl:for-each>
    		  </xsl:element>
    		</xsl:for-each>
    		
    		<xsl:if test="$trace='true'">
       			<xsl:message >About to build UserData</xsl:message>
        	</xsl:if>
        	
    		<xsl:call-template name="create_userdata">
    			<xsl:with-param name="userdata"><xsl:value-of select="concat($wbeobjectprefix,'_')"/></xsl:with-param>
    		</xsl:call-template>
	    
	    </xsl:element>
	    
	 </xsl:template> 
	 
	    
	 <xsl:template name="sensor_command">
	    
	    <xsl:if test="$trace='true'">
       				<xsl:message >About to build Sensor Command</xsl:message>
        </xsl:if>
        
        <xsl:element name="connector-bundle" >
	          <xsl:attribute name="name"><xsl:value-of select="$wbeevent"/></xsl:attribute>
	          <xsl:attribute name="type">Event</xsl:attribute>  
	        
	        <xsl:apply-templates select="parent::node()" />
	        
	        <xsl:if test="$trace='true'">
       				<xsl:message >About to build Header</xsl:message>
        	</xsl:if>
    		<xsl:call-template name="create_header">
    			<xsl:with-param name="header"><xsl:value-of select="concat($wbeobjectprefix,'_')"/></xsl:with-param>
    		</xsl:call-template>
    		
    		<xsl:if test="$trace='true'">
       				<xsl:message >About to build Command</xsl:message>
        	</xsl:if>
    		<xsl:variable name = "command" select="concat($wbeevent,'_Command')" />
    		<xsl:for-each select="wse:command">
    		  <xsl:element name = "{$command}" >  
    			<xsl:copy-of select="*" />
    			<xsl:for-each select="wse:field">
	          	 	<xsl:call-template name="insert_field"/>  
	        	</xsl:for-each>
    		  </xsl:element>
    		</xsl:for-each>
    		
    		
    		<xsl:call-template name="create_userdata">
    			<xsl:with-param name="userdata"><xsl:value-of select="concat($wbeobjectprefix,'_')"/></xsl:with-param>
    		</xsl:call-template>
    		
        </xsl:element>
        
    </xsl:template> 
    
    <xsl:template match="wse:sensor_events">
   
	     <xsl:variable name = "GlobalEvent" select="concat($wbeevent,'_Global')" />
	     <xsl:variable name = "GlobalObjPrefix" select="concat($wbeobjectprefix,'_Global')" />
	          
	   	 <xsl:call-template name="create_header">
	   		<xsl:with-param name="header"><xsl:value-of select="$GlobalObjPrefix"/></xsl:with-param>
	   	 </xsl:call-template>
	   	
	   	 <xsl:call-template name="create_userdata">
	   		<xsl:with-param name="userdata"><xsl:value-of select="$GlobalObjPrefix"/></xsl:with-param>
	   	 </xsl:call-template> 
          
    </xsl:template> 	
    
    <xsl:template name="create_header">
    	<xsl:param name="header"></xsl:param>
    	
    	<xsl:variable name="temp" > <xsl:value-of select = "$header" />  </xsl:variable>
    	<xsl:variable name = "realheader" select="concat($temp,'Header')" />
    	
    	<xsl:element name = "{$realheader}" > 
    		  <xsl:if test="wse:event_id/."> 
	          	<wse:event_id type="String"><xsl:value-of select="wse:event_id/."/></wse:event_id> 
	          </xsl:if>
	           <!-- We have to convert the date format from DC YYYY-MM-DDTHH:MM:SS.MSZ to YYYY-MM-DDTHH:MM:SSZ for WBE-->  	
	          <xsl:if test="wse:event_time/.">
	            <xsl:variable name = "tempdate"><xsl:value-of select="wse:event_time/."/></xsl:variable>
	            <xsl:choose>
	            	<xsl:when test="contains($tempdate,'.')">
	            	  <xsl:variable name = "tempdate1"><xsl:value-of select = "substring-before($tempdate,'.')" /></xsl:variable>
    				  <xsl:variable name = "wbedate"><xsl:value-of select = "concat($tempdate1,'Z')" /></xsl:variable>
	            	  <wse:event_time type="DateTime"><xsl:value-of select="$wbedate"/></wse:event_time>
	            	</xsl:when>
	                <xsl:otherwise>
	                  <wse:event_time type="DateTime"><xsl:value-of select="wse:event_time/."/></wse:event_time>
	                </xsl:otherwise>            
	            </xsl:choose>          
	          </xsl:if>
	          <xsl:if test="wse:event_type/.">
      		  	<wse:event_type type="String"><xsl:value-of select="wse:event_type/."/></wse:event_type>
      		  </xsl:if>
      		  <xsl:if test="wse:source_id/.">
      		  	<wse:source_id type="String"><xsl:value-of select="wse:source_id/."/></wse:source_id>
      		  </xsl:if>
      		  <xsl:if test="wse:target_id/.">
      		  	<wse:target_id type="String"><xsl:value-of select="wse:target_id/."/></wse:target_id>
      		  </xsl:if>
      		  <xsl:if test="wse:originating_event_id/.">
      		  	<wse:originating_event_id type="String"><xsl:value-of select="wse:originating_event_id/."/></wse:originating_event_id>
	          </xsl:if>
	          <xsl:if test="wse:priority/.">
	          	<wse:priority type="Integer"><xsl:value-of select="wse:priority/."/></wse:priority>
	          </xsl:if>
	          <xsl:for-each select="wse:field">
	          	 <xsl:call-template name="insert_field"/>  
	          </xsl:for-each>
	    </xsl:element>
	    
	</xsl:template>
	
	 <xsl:template name="create_userdata">
        <xsl:param name="userdata"></xsl:param>
    	
    	<xsl:variable name="temp" > <xsl:value-of select = "$userdata" />  </xsl:variable>
    	<xsl:variable name = "realuserdata" select="concat($temp,'UserData')" />
    	
       	<xsl:for-each select="wse:user_data">
       	
       	  <xsl:if test="$transform_for_rules='true'">
       	  		 <xsl:if test="$trace='true'">
       				<xsl:message >transform_for_rules is true, calling "insert_rule_field" template </xsl:message>
        		</xsl:if>
        
       	  		<xsl:for-each select="wse:field">   
       				<xsl:call-template name="insert_rule_field">
       				<xsl:with-param name="eName"><xsl:value-of select="$realuserdata"/></xsl:with-param>
       				</xsl:call-template>
       			</xsl:for-each>
        	</xsl:if>
        	
        <xsl:if test="$transform_for_rules='false'">   
         	<xsl:if test="$trace='true'">
       			<xsl:message >transform_for_rules is false, calling "insert_field" template </xsl:message>
        	</xsl:if>     		
       	  <xsl:element name = "{$realuserdata}" >
   			<xsl:for-each select="wse:field">   			 
       				<xsl:call-template name="insert_field"/>         		 
	        </xsl:for-each>
	        
   		  </xsl:element>
   		  </xsl:if> 
    	</xsl:for-each>
    	
    	   
	</xsl:template>
	
	<xsl:template name="insert_field">
	  	<xsl:if test="$trace='true'">
       		<xsl:message >inside insert_field - template </xsl:message>
        </xsl:if>
        <xsl:variable name="field_name" > <xsl:value-of select="concat(@wse:prefix,':',translate(@wse:name,' ','_'))"/> </xsl:variable>
        <xsl:variable name="field_namespace" ><xsl:value-of select="@wse:namespace" /> </xsl:variable>
        <xsl:variable name="field_namespace_prefix" ><xsl:value-of select="@wse:prefix" /> </xsl:variable>

        <xsl:element name="{$field_name}" namespace="{$field_namespace}" >
         		<xsl:attribute name="type"><xsl:value-of select="@type"/></xsl:attribute>
         		<xsl:value-of select="." />
        </xsl:element>
       
	    
	</xsl:template>
	
	<xsl:template name="insert_rule_field">		
		<xsl:param name="eName"></xsl:param>
	  	<xsl:if test="$trace='true'">
       		<xsl:message >inside insert_rule_field - template </xsl:message>
        </xsl:if>
        <xsl:variable name="field_name" > <xsl:value-of select="@wse:name"/> </xsl:variable>
        <xsl:variable name="field_namespace" ><xsl:value-of select="@wse:namespace" /> </xsl:variable>
        
       <xsl:variable name="temp" > <xsl:value-of select = "Asset_Observation_" />  </xsl:variable>
    	<xsl:variable name = "realuserdata" select="concat($temp,'UserData')" />    	
    	
       <xsl:element name ="{$eName}" >
        <xsl:element name="name" namespace="{$field_namespace}" >
         		<xsl:value-of select="@wse:name" />
        </xsl:element>
        <xsl:element name="value" namespace="{$field_namespace}" >
         		<xsl:value-of select="." />
        </xsl:element>
       </xsl:element>	    
	</xsl:template>
    
</xsl:stylesheet>

